from .data import langs


class Language(object):
    def __init__(self, iso):
        self.iso = iso
        self.name = None

        try:
            self.name = langs[iso]
        except KeyError:
            raise KeyError('Language with iso code \'%s\' unknown' % iso)

    def __repr__(self):
        return u'Language(iso={})'.format(self.iso)

    def __str__(self):
        return self.name



import treelib

from .data import etyms as etymwn_data
from .language import Language


class Word(object):
    def __init__(self, word, language='eng'):
        if not isinstance(word, ("".__class__, u"".__class__)):
            raise ValueError('word must be a string')
        self.word = word
        self.language = Language(language)
        self._origins = None
        self._id = u"{}:{}".format(self.word, self.language.iso)

    def origins(self, recursive=False):
        if self._origins:
            return self._origins

        row = list(filter(
            lambda entry: entry['a_word'] == self.word and entry[
                'a_lang'] == self.language.iso, etymwn_data))

        result = [Word(item['b_word'], item['b_lang']) for item in row]

        if recursive:
            for origin in result:
                for child in origin.origins():
                    # Check word isn't already in tree before appending
                    if child not in result and child != self:
                        result.append(child)

        self._origins = result
        return self._origins

    def tree(self):
        ety_tree = treelib.Tree()

        word_obj = Word(self.word, self.language.iso)
        root = word_obj.pretty
        root_key = self._id

        # Create parent node
        ety_tree.create_node(root, root_key, data=self)

        # Add child etymologies
        self._tree(ety_tree, root_key)

        return ety_tree

    def _tree(self, tree_obj, parent):
        source_word = Word(self.word, self.language.iso)
        word_origins = source_word.origins()

        for origin in word_origins:
            key = origin._id
            # Recursive call to add child origins
            if self.word == origin.word:
                continue

            try:
                tree_obj.create_node(
                    origin.pretty, key, parent=parent, data=origin
                )
            except treelib.exceptions.DuplicatedNodeIdError:
                continue
            origin._tree(tree_obj, key)

    @property
    def pretty(self):
        return u"{word} ({lang})".format(
            word=self.word,
            lang=self.language.name)

    def __lt__(self, other):
        if isinstance(other, Word):
            return self.pretty < other.pretty
        return self.pretty < other

    def __eq__(self, other):
        if isinstance(other, Word):
            return self._id == other._id
        return self.pretty == other

    def __str__(self):
        return self.pretty

    def __repr__(self):
        return u'Word({word}, language={lang})'.format(
            word=self.word, lang=self.language
        )



from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ety',
    version='1.1.0',
    description='find the etymological origins of a word',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jmsv/ety-python',
    author='James Vickery',
    author_email='dev@jamesvickery.net',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4',
    keywords='etymology origins english language words',
    packages=['ety'],
    install_requires=[
        "treelib",
    ],
    extras_require={
        'dev': ['flake8'],
    },
    package_data={
        'ety': ['wn/etymwn-relety.json', 'wn/iso-639-3.json'],
    },
    entry_points={
        'console_scripts': [
            'ety=ety:cli',
        ],
    },
    project_urls={
        'Source': 'https://github.com/jmsv/ety-python',
        'Bug Reports': 'https://github.com/jmsv/ety-python/issues',
    },
)



# encoding: utf-8

import io
import json

from pkg_resources import resource_filename


def load_relety():
    resource = resource_filename('ety', 'wn/etymwn-relety.json')
    with io.open(resource, 'r', encoding='utf-8') as f:
        return json.load(f)


def load_country_codes():
    resource = resource_filename('ety', 'wn/iso-639-3.json')
    with io.open(resource, 'r', encoding='utf-8') as f:
        return json.load(f)

    # loaded_langs = []
    # resource = resource_filename('ety', 'wn/iso-639-3.json')
    # with io.open(resource, 'r', encoding='utf-8') as f:
    #     countries_json = json.load(f)
    # for country in countries_json:
    #     loaded_langs.append({
    #         'name': country['name'],
    #         'iso6393': country['iso6393'],
    #     })
    # return loaded_langs


etyms = load_relety()
langs = load_country_codes()



import argparse
from random import choice

from . import data
from .word import Word, Language  # noqa: F401


def cli():
    parser = argparse.ArgumentParser(prog="ety")
    parser.add_argument("words", type=str, nargs='+',
                        help="the search word(s)")
    parser.add_argument("-r", "--recursive", help="search origins recursively",
                        action="store_true")
    parser.add_argument("-t", "--tree", help="display etymology tree",
                        action="store_true")
    args = parser.parse_args()

    for word in args.words:
        roots = origins(word, recursive=args.recursive)

        if not roots:
            print("No origins found for word: {}".format(word))
            continue

        if args.tree:
            result = str(tree(word)).strip()
        else:
            result = '\n'
            if word is args.words[0]:
                result = ''
            result += '\033[1m' + word + '\033[0m \n \u2022 '
            result += '\n \u2022 '.join(root.pretty for root in roots)

        print(result)

    return 0


def _get_source_word(word, word_lang):
    if isinstance(word, Word):
        return word
    return Word(word, word_lang)


def origins(word, word_lang='eng', recursive=False):
    source_word = _get_source_word(word, word_lang)
    return source_word.origins(recursive)


def tree(word, word_lang='eng'):
    source_word = _get_source_word(word, word_lang)
    return source_word.tree()


def random_word(lang='eng'):
    row = list(filter(lambda entry: entry['a_lang'] == lang, data.etyms))
    word = choice(row)['a_word']
    return Word(word, lang)
